import numpy as np
import matplotlib.pyplot as plt
import matplotlib
from scipy.stats import binned_statistic
#from Binning_scipy import bin_data_given_bin_center
from glob import glob1
import os
#import addcopyfighandler
import glob

matplotlib.rcParams.update({'font.size': 30})
matplotlib.rcParams['lines.linewidth'] = 3
matplotlib.rcParams['font.family'] = 'serif'
#matplotlib.rcParams['text.usetex'] = False
#matplotlib.rcParams['text.latex.unicode'] = False
params = {'legend.edgecolor': '0',
          'legend.borderaxespad': 1.5,
          'legend.fancybox': False,
          'legend.fontsize': 16.0,
          'legend.framealpha': 0.5,
          'legend.labelspacing': 0.3,
          'legend.markerscale': 1.0,
          'figure.figsize': (4, 4),
          'font.family' : 'sans-serif',
          'figure.subplot.left':0.16,
          'figure.subplot.bottom': 0.16,
          'axes.labelsize': 16,
          'axes.titlesize': 25,
          'text.usetex': False,
          'svg.fonttype': 'none',
          'axes.linewidth': 3,
          'axes.xmargin': 0.03,
          'axes.ymargin': 0.03,
          'xtick.direction': 'in',
          'xtick.labelsize': 16,
          'xtick.major.pad': 10,
          'xtick.major.size': 10,
          'xtick.major.width': 3,
          'xtick.minor.pad': 10,
          'xtick.minor.size': 5,
          'xtick.minor.visible': True,
          'xtick.minor.width': 2,
          'xtick.top': True,
          'ytick.direction': 'in',
          'ytick.labelsize': 16,
          'ytick.major.pad': 10,
          'ytick.major.size': 10,
          'ytick.major.width': 3,
          'ytick.minor.pad': 10,
          'ytick.minor.size': 5,
          'ytick.minor.visible': True,
          'ytick.minor.width': 2,
          'ytick.right': True, }
plt.rcParams.update(params)


def bin_edge2_bin_center(bin_edges):
    center = np.zeros(len(bin_edges)-1)
    for i in range(len(bin_edges)-1):
        center[i] = bin_edges[i] + (bin_edges[i+1]-bin_edges[i])/2
    return center

def bin_center2_bin_edge(bin_center):
    edges = np.zeros(len(bin_center)+1)
    bin_sep = (bin_center[1]-bin_center[0])/2  # assume fixed bin width
    for i in range(len(bin_center)):
        #         print (i,bin_center[i])
        edges[i] = bin_center[i] - bin_sep
    edges[-1] = bin_center[-1] + (bin_center[-1]-bin_center[-2])/2
    return edges


def bin_data_given_bin_center(x, y, bin_center=[], nbin=10, Range=None, opt='mean'):
    '''
    Bin data, specifying the bin center
    arg :
    1. x : xdata
    2. y : ydata
    3. bin_center = an array of bin_center (default is None)
    4. nbin = 10 if the bin_center is None
    5. Range = None , given in the form (min(x),max(x))
    6. opt = sum (for statistics)
    count : compute the count of points within each bin. This is identical to an unweighted histogram. values array is not referenced.
    sum : compute the sum of values for points within each bin. This is identical to a weighted histogram.
    mean: compute the mean of the y-values in the same bin: useful for weighted data.
    std: compute the error bar
    Return:
    1. bin_center
    2. bin_counter
    '''

    if len(bin_center) != 0:
        bin_edges = bin_center2_bin_edge(bin_center)
    else:
        bin_edges = np.linspace(min(x), max(x), nbin)

    ret = binned_statistic(x, y, statistic=opt, bins=bin_edges, range=Range)
    return bin_edge2_bin_center(ret.bin_edges), ret.statistic



import scipy


directory = 'C:/Users/photonics/Dropbox (Heriot-Watt University Team)/RES_EPS_Quantum_Photonics_Lab/' \
            'Experiments/Current Experiments/SiC_Vanadium/No_Isotopically_Purified_Sample/Charge_State/' \
            'Experiment_1_Only_Green_Pulsed_CW_Power_Dep/QWP_83_Linear_Polarized_Ex/'

txt_files = glob.glob(directory+"/*.txt")
print(txt_files)
print(txt_files[1])

decay_time=[]
decay_time_std=[]


x = np.loadtxt(txt_files[5])
y = np.loadtxt(txt_files[4])

file_name = txt_files[5]
name_parts = file_name.split("_")

# extract the relevant portion of the file name based on its position in the list
info_string = name_parts[27]

print(info_string)
binwidth=5

binx,biny = bin_data_given_bin_center(x,y,nbin=int(len(x)/binwidth),opt='mean')
biny -= np.min(biny)
max_y = np.max(biny)

# Normalize the data as a percentage of the maximum value
normalized_y = (biny / max_y)

start_point = 87

minus=start_point
end_point=start_point+400

# plt.clf()
# plt.plot(binx, biny, label="Delay = 0 ms")
# plt.plot(binx[start_point:],biny[start_point:])
# plt.title(f"Red power: {info_string}")
#plt.show()

def monoExp(x, m, t, b):
    return m * np.exp(-x/t) + b
plt.clf()
p0 = (500, 230, 100) # start with values near those we expect
params, cv = scipy.optimize.curve_fit(monoExp, (binx[start_point:end_point] )/1e9, normalized_y[start_point:end_point], p0)
m, t, b = params

color_rgb = (255, 125, 13)
color_mpl = tuple(x/255 for x in color_rgb)

plt.plot((binx[0:end_point])/1e9, normalized_y[0:end_point], '.',  color= color_mpl , label="Sample B")
plt.plot((binx[start_point:end_point])/1e9, monoExp((binx[start_point:end_point])/1e9, m, t, b),'--', color= color_mpl, linewidth=4)
plt.title(f"Y ="+ str(format(m,".0f")) +"*e^(-x/"+str(format(t,".2f"))+ ")+"+ str(format(b,".0f")) + f" Red power: {info_string}")
#plt.title('tau=' + str(format(fitted_t,".2f")) + ' +-'+ str(format(1/sigma[1],".2f"))+' ms' + ' & Amp =' +str(format(m,".2f")) + ' +-'+ str(format(sigma[0],".2f"))+' cnts' )
plt.xlabel('Time Delay (ms)')
plt.ylabel('Photon counts (a.u.)')

sigma = np.sqrt([cv[0,0], cv[1,1], cv[2,2]])
print(sigma)
print(cv)
print(t)
fitted_t= t
sig_t=sigma[1]
print('Fitted tau in ms', fitted_t, ' plus minus', sig_t)
#plt.text(700, 230, r'T fit=' + str(format(t,".2f"))+ u"\u00B1" + str(format(sig_t,".2f")) +' ms')
plt.legend()
#plt.show()
plt.minorticks_off()
plt.tick_params(axis='both', which='major', direction='out')
plt.tick_params(axis='both', which='major', top=False, right=False)


directory = 'C:/Users/photonics/Dropbox (Heriot-Watt University Team)/RES_EPS_Quantum_Photonics_Lab/' \
            'Experiments/Current Experiments/SiC_Vanadium/Pulsed_Measurements/Linearly_Polarized_Excitation/' \
            'First_Exp_Green_Pulsed_Telecom_PD/Measurement_2_Old_Swabian/'

x = np.loadtxt(directory + '2022_09_15_11_29_D_Lucky_2b_234.42480THz_6uW_Green_14uW_1000Gauss_4_3K_n_pulses_4000_binwidth_1ms_Time.txt')
y = np.loadtxt(directory+ '2022_09_15_11_29_D_Lucky_2b_234.42480THz_6uW_Green_14uW_1000Gauss_4_3K_n_pulses_4000_binwidth_1ms_Counts.txt')


binwidth=5

binx,biny = bin_data_given_bin_center(x,y,nbin=int(len(x)/binwidth),opt='mean')

biny -= np.min(biny)
max_y = np.max(biny)

# Normalize the data as a percentage of the maximum value
normalized_y = (biny / max_y)


print(len(binx))
start_point = 89

minus=start_point
end_point=start_point+400

#plt.clf()
#plt.plot(binx, biny, label="Delay = 0 ms")
#plt.plot(binx[start_point:],biny[start_point:])
#plt.title(f"Red power: {info_string}")
#plt.show()

def monoExp(x, m, t, b):
    return m * np.exp(-x/t) + b
#plt.clf()

color_rgb = (122, 13, 255)
color_mpl = tuple(x/255 for x in color_rgb)

p0 = (1150, 70, 400) # start with values near those we expect
params, cv = scipy.optimize.curve_fit(monoExp, (binx[start_point:end_point])/1e9, normalized_y[start_point:end_point], p0)
m, t, b = params
plt.plot((binx[0:end_point])/1e9, normalized_y[0:end_point], '.', color= color_mpl, label="Sample A")
plt.plot((binx[start_point:end_point])/1e9, monoExp((binx[start_point:end_point])/1e9, m, t, b),'--',color= color_mpl, linewidth=4)
plt.title(f"Y ="+ str(format(m,".0f")) +"*e^(-x/"+str(format(t,".2f"))+ ")+"+ str(format(b,".0f")) + f" Red power: {info_string}")
#plt.title('tau=' + str(format(fitted_t,".2f")) + ' +-'+ str(format(1/sigma[1],".2f"))+' ms' + ' & Amp =' +str(format(m,".2f")) + ' +-'+ str(format(sigma[0],".2f"))+' cnts' )
plt.xlabel('time (ms)')
plt.ylabel('intensity (a.u.)')

sigma = np.sqrt([cv[0,0], cv[1,1], cv[2,2]])
print(sigma)
print(cv)
print(t)
fitted_t= t
sig_t=sigma[1]
print('Fitted tau in ms', fitted_t, ' plus minus', sig_t)
#plt.text(700, 230, r'T fit=' + str(format(t,".2f"))+ u"\u00B1" + str(format(sig_t,".2f")) +' ms')
plt.legend()
plt.minorticks_off()
plt.tick_params(axis='both', which='major', direction='out')
plt.tick_params(axis='both', which='major', top=False, right=False)
plt.show()
